@@ -1,5 +1,6 @@
 #! /usr/bin/env python
-""" Resets the 10GbE cores on all X engines through fabric rst port toggle.
+"""
+Resets the 10GbE cores on all X engines through fabric rst port toggle.
 """
 import corr, time, sys, numpy, os, logging
 
@@ -28,15 +29,16 @@ if __name__ == '__main__':
 
     opts, args = p.parse_args(sys.argv[1:])
 
-    if args==[]:
-        config_file=None
+    if args == []:
+        config_file = None
     else:
-        config_file=args[0]
-    verbose=opts.verbose
+        config_file = args[0]
+    #verbose=opts.verbose
+    verbose = False
 
 try:    
     print 'Connecting...',
-    c=corr.corr_functions.Correlator(config_file=config_file,log_level=logging.DEBUG if verbose else logging.INFO, connect=False)
+    c = corr.corr_functions.Correlator(config_file = config_file, log_level = logging.DEBUG if verbose else logging.INFO, connect = False)
     c.connect()
     print 'done'
 
@@ -45,15 +47,15 @@ try:
 
     #DO NOT RESET THE 10GBE CORES SYNCHRONOUSLY!
     if c.config['feng_out_type']=='10gbe':
-        c.feng_ctrl_set_all(gbe_enable=False, gbe_rst=False)
-        c.feng_ctrl_set_all(gbe_enable=False,gbe_rst=True)
-        c.feng_ctrl_set_all(gbe_enable=False,gbe_rst=False)
-        c.feng_ctrl_set_all(gbe_enable=True,gbe_rst=False)
+        c.feng_ctrl_set_all(gbe_enable = False, gbe_rst = False)
+        c.feng_ctrl_set_all(gbe_enable = False, gbe_rst = True)
+        c.feng_ctrl_set_all(gbe_enable = False, gbe_rst = False)
+        c.feng_ctrl_set_all(gbe_enable = True,  gbe_rst = False)
 
-    c.xeng_ctrl_set_all(gbe_enable=False, gbe_rst=False) 
-    c.xeng_ctrl_set_all(gbe_enable=False, gbe_rst=True)
-    c.xeng_ctrl_set_all(gbe_enable=False, gbe_rst=False)
-    c.xeng_ctrl_set_all(gbe_enable=True, gbe_rst=False)
+    c.xeng_ctrl_set_all(gbe_enable = False, gbe_rst = False) 
+    c.xeng_ctrl_set_all(gbe_enable = False, gbe_rst = True)
+    c.xeng_ctrl_set_all(gbe_enable = False, gbe_rst = False)
+    c.xeng_ctrl_set_all(gbe_enable = True,  gbe_rst = False)
 
     print 'done.'
 
