@@ -1,4 +1,5 @@
 import logging
+from corr import termcolors
 
 class DebugLogHandler(logging.Handler):
     """A logger for KATCP tests."""
@@ -27,9 +28,17 @@ class DebugLogHandler(logging.Handler):
     def printMessages(self):
         for i in self._records:
             if i.exc_info:
-                print '%s: %s Exception: '%(i.name,i.msg),i.exc_info[0:-1]
+                print termcolors.colorize('%s: %s Exception: '%(i.name,i.msg),i.exc_info[0:-1],fg='red')
             else:    
-                print '%s: %s'%(i.name,i.msg)
+                if i.levelno < logging.WARNING: 
+                    print termcolors.colorize('%s: %s'%(i.name,i.msg),fg='green')
+                elif (i.levelno >= logging.WARNING) and (i.levelno < logging.ERROR):
+                    print termcolors.colorize('%s: %s'%(i.name,i.msg),fg='yellow')
+                elif i.levelno >= logging.ERROR: 
+                    print termcolors.colorize('%s: %s'%(i.name,i.msg),fg='red')
+                else:
+                    print '%s: %s'%(i.name,i.msg)
+
 
 #log_handler = TestLogHandler()
 #logging.getLogger("katcp").addHandler(log_handler)
